/*
 * App.cpp
 *
 * This file takes care of launching the App and processing its core messages,
 * and some interface work.
 *
 * Copyright (c) Malcolm Smith 2001-2007.  No warranty is provided.
 */

#include "pch.h"

HINSTANCE_T hInst;

DWORD WinVer;

HINSTANCE_T GethInst()
{
	return hInst;
}

int MplayProcessCmd(LPSTR lpCmdLine)
{
#ifdef UNICODE
	LPTSTR UniCmdLine;

	UniCmdLine = (LPTSTR)malloc((strlen(lpCmdLine) + 1) * sizeof(TCHAR));

	if (UniCmdLine == NULL) {
		//
		// We're probably going to crash anyway, but abort
		// the command line parsing.
		//
		return MPLAY_ERROR_NO_MEMORY;
	}

	OemToChar( lpCmdLine, UniCmdLine );

	SendMessage(GethWndMain(), C_WM_CMDLINE, 0, (LPARAM)UniCmdLine);

	free(UniCmdLine);

#else
	PostMessage(GethWndMain(), C_WM_CMDLINE, 0, (LPARAM)lpCmdLine);
#endif
	return MPLAY_ERROR_SUCCESS;
}

BOOL MplayInitApplication(HINSTANCE_T hInstance)
{
	WNDCLASSEX wc;

	LPCTSTR SmallIcon;

	//
	// If we go to high priority, this should protect against audio
	// crappiness when the system is under load.  We might not be
	// allowed to do this, so do it blind and ignore errors.
	//
	SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);

	WinVer = GetVersion();

	//
	//  This is a horrible hack.  I can't see a better approach.
	//  Win95 etc use a blue title bar with white text, so a white
	//  icon looks better.  Vista uses a really pale title bar, so
	//  the black icon looks better.  I'd base this choice off the
	//  title bar, but the same icon is also used in the taskbar.
	//  Worse, there's no guarantee the same icon can be right in
	//  both places.  On the other hand, most apps only have *one*
	//  icon, so we can do better than that ;).
	//

	if (MPLAY_WINVER_MAJOR(WinVer) >= 6) {
		SmallIcon = MAKEINTRESOURCE( IDI_ICOBLACK );
	} else {
		SmallIcon = MAKEINTRESOURCE( IDI_ICOWHITE );
	}

	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)MplayMainWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hCursor = LoadCursor( NULL, IDC_ARROW );
	wc.hbrBackground = (HBRUSH) GetStockObject( LTGRAY_BRUSH );
	wc.lpszMenuName =  _T("MPMENU");
	wc.lpszClassName = _T("MalxMediaPlayer");
	wc.hIcon = LoadIcon(hInstance, SmallIcon);
	wc.cbSize = sizeof(wc);
	wc.hIconSm = LoadImage(hInstance,
		SmallIcon,
		IMAGE_ICON,
		16,
		16,
		LR_DEFAULTCOLOR);
	return RegisterClassEx(&wc);
}


int WINAPI
WinMain(HINSTANCE_T hInstance,HINSTANCE_T hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
	MSG msg;
	HWND hWnd;
	HACCEL acctable;

	hInst = hInstance;
	
	srand((UINT)(time(NULL)/256^(sizeof(time_t)-sizeof(UINT))));
	if (!MplayInitApplication(hInstance)) {
		return FALSE;
	}
	if (MplayCreateAppWindows(hInstance, nCmdShow) != MPLAY_ERROR_SUCCESS) {
		return FALSE;
	}
	acctable = LoadAccelerators(hInstance, _T("KEYS"));
	hWnd = GethWndMain();
	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);
	SendMessage(hWnd, C_WM_REALBEGIN, 0, 0);
	MplayProcessCmd(lpCmdLine);	
	while (GetMessage(&msg, NULL, 0, 0)) {
		if (!TranslateAccelerator(hWnd, acctable, &msg)) {
			TranslateMessage(&msg);
		}
		DispatchMessage(&msg);
	}
	return EXIT_SUCCESS;
}


LRESULT APIENTRY
MplayMainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static LIST List;
	static LIST RecentSearches;
	static DWORD dwFindFlags;
	static TCHAR szOpenFile[MAX_PATH];
	static DWORD dwRepeat;
	static BOOL bFullPath;
	static BOOL bOnTop;
	TCHAR  szTemp[MAX_PATH];

	switch (message) {
 	case WM_COMMAND:
		switch(HIWORD(wParam)) {
		case LBN_SELCANCEL:
			MplayCloseStream(hWnd);
			break;

		case LBN_DBLCLK:
			{
				int nIndex;
				nIndex = MplayGetSelFile(GethWndCli());
				if (nIndex >= 0) {
					MplayPlayMediaFile(hWnd, &List, nIndex);
				}
				MplayUpdatePauseStatus(hWnd, FALSE);
			}
			break;
		}
		switch(LOWORD(wParam)) {
			/*************/
			/* FILE MENU */
			/*************/
			case IDM_NEW:
				_tcscpy(szOpenFile, _T(""));
				MplayRefreshTitle(szOpenFile);
				MplayClearList(&List);
				MplayRefreshList(GethWndCli(),
					&List,
					bFullPath);
				break;
			case IDM_OPEN:
				_tcscpy(szTemp, _T(""));
				if (MplayGetaFile(szTemp,FALSE) == MPLAY_ERROR_SUCCESS) {
					LPTSTR a;
					MplayClearList(&List);
					MplayInputFile(&List,
						szTemp);
					MplayRefreshList(GethWndCli(),
						&List,
						bFullPath);
					_tcscpy(szOpenFile,szTemp);
					_tcsupr(szTemp);
					if ((a = _tcsrchr(szTemp, _T('.'))) != NULL) {
						if (_tcscmp(a+1, _T("M3U"))) {
							_tcscpy(szOpenFile, _T(""));
						}
					} else _tcscpy(szOpenFile, _T(""));
					MplayRefreshTitle(szOpenFile);
				}
				break;
			case IDM_SAVE:
				if (_tcslen(szOpenFile)) {
					MplayWriteList(&List,szOpenFile);
				} else {
					MessageBeep(MB_ICONEXCLAMATION);
					MessageBox(hWnd,
						_T("There is currently no open file."),
						_T(MPLAY_NAME),
						MB_ICONEXCLAMATION);
				}
				break;

			case IDM_SAVEAS:
				_tcscpy(szTemp, _T(""));
				if (MplayGetaFile(szTemp,TRUE) == MPLAY_ERROR_SUCCESS) {
					_tcscpy(szOpenFile, szTemp);
					MplayRefreshTitle(szOpenFile);
					MplayWriteList(&List,szTemp);
				}
				break;

			case IDM_EXIT:
				DestroyWindow(hWnd);
				PostQuitMessage(0);
				break;
			case IDM_ABOUT:
				{
					_stprintf(szTemp, _T("Malx %s, %i.%i.%i.")
#ifdef PRERELEASE
						_T(" Prerelease build, built on %s.")
#endif
						_T(" %s.  ")
						_T("This software is licensed under the terms of the GNU Public License, ")
						_T("version 2 or any later version."),
						_T(MPLAY_NAME),
						MPLAY_VER_MAJOR,
						MPLAY_VER_MINOR,
						MPLAY_VER_MICRO,
#ifdef PRERELEASE
						_T(__DATE__),
#endif
						_T(MPLAY_COPYRIGHT)
						);
					MessageBox(hWnd,
						szTemp,
						_T(MPLAY_NAME),
						MB_ICONINFORMATION);
					break;
				}
#ifdef DEBUG_MPLAY
			case IDM_DEBUG:
				ASSERT( !"User breakin request.  This is NOT a bug." );
				break;
#endif
			/**************/
			/* MEDIA MENU */
			/**************/
			case IDM_PLAY:
				{
					int nIndex;
					nIndex = MplayGetSelFile(GethWndCli());
					if (nIndex >= 0) {
						MplayPlayMediaFile(hWnd,
							&List,
							nIndex);
					}
					MplayUpdatePauseStatus(hWnd, FALSE);
				}
				break;
			case IDM_STOP:
				MplayCloseStream(hWnd);
				MplayUpdatePauseStatus(hWnd, FALSE);
				break;
			case IDM_PAUSE:
				MplayPauseAudio(hWnd);
				MplayUpdatePauseStatus(hWnd, TRUE);
				break;
			case IDM_RESUME:
				MplayResumeAudio(hWnd);
				MplayUpdatePauseStatus(hWnd, FALSE);
				break;

			case IDM_NEXT:
				MplayPlayMediaFile(hWnd, &List, PLAY_NEXT);
				break;
			case IDM_PREVIOUS:
				MplayPlayMediaFile(hWnd, &List, PLAY_PREV);
				break;
			case IDM_FIRST:
				MplayPlayMediaFile(hWnd, &List, 0);
				break;
			case IDM_LAST:
				ASSERT( GetListLength(&List) < sizeof(INT_MAX) );
				MplayPlayMediaFile(hWnd,
					&List,
					(int)GetListLength(&List)-1);
				break;
			case IDM_REP_NEXTTRACK:
				dwRepeat = REPEAT_NEXTTRACK;

				CheckMenuItem(GetMenu(hWnd),
					IDM_REP_NEXTTRACK,
					MF_CHECKED);

				CheckMenuItem(GetMenu(hWnd),
					IDM_REP_THISTRACK,
					MF_UNCHECKED);

				CheckMenuItem(GetMenu(hWnd),
					IDM_REP_NOTRACK,
					MF_UNCHECKED);
				break;
			case IDM_REP_THISTRACK:
				dwRepeat = REPEAT_THISTRACK;
				CheckMenuItem(GetMenu(hWnd),
					IDM_REP_NEXTTRACK,
					MF_UNCHECKED);

				CheckMenuItem(GetMenu(hWnd),
					IDM_REP_THISTRACK,
					MF_CHECKED);

				CheckMenuItem(GetMenu(hWnd),
					IDM_REP_NOTRACK,
					MF_UNCHECKED);
				break;
			case IDM_REP_NOTRACK:
				dwRepeat = REPEAT_NOTRACK;
				CheckMenuItem(GetMenu(hWnd),
					IDM_REP_NEXTTRACK,
					MF_UNCHECKED);

				CheckMenuItem(GetMenu(hWnd),
					IDM_REP_THISTRACK,
					MF_UNCHECKED);

				CheckMenuItem(GetMenu(hWnd),
					IDM_REP_NOTRACK,
					MF_CHECKED);
				break;
			case IDM_VOL_UP:
				{
					DWORD Vol;
					if (MplayGetVolume(&Vol) == MPLAY_ERROR_SUCCESS) {
						Vol = ((Vol / 0x800) + 1) * 0x8;
						if (Vol > 0xff) Vol = 0xff;
						Vol = MAKEWORD(Vol, Vol);
						MplaySetVolume(Vol);
					} else {
						MessageBox(hWnd,
							_T("Could not change volume"),
							_T(MPLAY_NAME),
							MB_ICONEXCLAMATION);
					}
				}
				break;
			case IDM_VOL_DOWN:
				{
					DWORD Vol;
					if (MplayGetVolume(&Vol) == MPLAY_ERROR_SUCCESS) {
						Vol = (Vol / 0x800);
						if (Vol > 0) Vol = (Vol - 1) * 0x8;
						Vol = MAKEWORD(Vol, Vol);
						MplaySetVolume(Vol);
					} else {
						MessageBox(hWnd,
							_T("Could not change volume"),
							_T(MPLAY_NAME),
							MB_ICONEXCLAMATION);
					}
				}
				break;
			/**************/
			/* Items Menu */
			/**************/
			case IDM_SORTDISPLAY:
				MplaySortList(&List, SORT_DISPLAY);
				MplayRefreshList(GethWndCli(),
					&List,
					bFullPath);
				break;

			case IDM_SORTFILE:
				MplaySortList(&List, SORT_FILE);
				MplayRefreshList(GethWndCli(),
					&List,
					bFullPath);
				break;
			case IDM_SORTPATH:
				MplaySortList(&List, SORT_PATH);
				MplayRefreshList(GethWndCli(),
					&List,
					bFullPath);
				break;

			case IDM_SHUFFLE:
				MplayShuffleList(&List);
				MplayRefreshList(GethWndCli(),
					&List,
					bFullPath);
				break;
			case IDM_MOVEUP:
				{
					int i;
					i = (int)SendMessage(GethWndCli(),
						LB_GETCURSEL,
						0,
						0);
					if (i>0) {
						MplaySwapListItems(&List, i, i-1);
						if (SendMessage(GethWndCli(),LB_GETTEXT,(WPARAM)i,(LPARAM)szTemp)) {
							SendMessage(GethWndCli(),
								LB_DELETESTRING,
								(WPARAM)i,
								0);
							SendMessage(GethWndCli(),
								LB_INSERTSTRING,
								(WPARAM)i-1,
								(LPARAM)szTemp);
						} else {
							ASSERT( !"Refreshing list to perform move" );
							MplayRefreshList(GethWndCli(),
								&List,
								bFullPath);
						}
						SendMessage(GethWndCli(),
							LB_SETCURSEL,
							(WPARAM)i-1,
							0);
					}
				}
				break;
			case IDM_MOVEDOWN:
				{
					int i;
					i=(int)SendMessage(GethWndCli(), 
						LB_GETCURSEL,
						0,
						0);
					if (i >= 0) {
						if (i<(int)GetListLength(&List)-1) {
							MplaySwapListItems(&List,
								i,
								i+1);
							if (SendMessage(GethWndCli(),LB_GETTEXT,(WPARAM)i,(LPARAM)szTemp)) {						
								SendMessage(GethWndCli(),
									LB_DELETESTRING,
									(WPARAM)i,
									0);
								SendMessage(GethWndCli(),
									LB_INSERTSTRING,
									(WPARAM)i+1,
									(LPARAM)szTemp);
							} else {
								ASSERT( !"Refreshing list to perform move" );
								MplayRefreshList(GethWndCli(),
									&List,
									bFullPath);
							}
							SendMessage(GethWndCli(),LB_SETCURSEL,(WPARAM)i+1,0);
						}
					}
				}
				break;
			case IDM_DELETE:
				{
					DWORD i;
					i=(DWORD)SendMessage(GethWndCli(),
						LB_GETCURSEL,
						0,
						0);
					if (i>=0) {
						MplayDeleteListItem(&List, i);
						SendMessage(GethWndCli(),
							LB_DELETESTRING,
							(WPARAM)i,
							0);
						if (i==GetListLength(&List)) i--;
						SendMessage(GethWndCli(),
							LB_SETCURSEL,
							(WPARAM)i,
							0);
					}
				}
				break;
			case IDM_DISPPATH:
				bFullPath=!bFullPath;
				MplayRefreshList(GethWndCli(),
					&List,
					bFullPath);
				if (bFullPath) {
					CheckMenuItem(GetMenu(hWnd),
						IDM_DISPPATH,
						MF_CHECKED);
				} else {
					CheckMenuItem(GetMenu(hWnd),
						IDM_DISPPATH,
						MF_UNCHECKED);
				}
				break;
			case IDM_ONTOP:
				bOnTop=!bOnTop;
				MplaySetOnTop(hWnd, bOnTop);
				break;
			case IDM_DISPTRACK:
				MplaySetTracker(hWnd, !MplayIsTracker());
				break;
			case IDM_REFRESH:
				MplayRefreshList(GethWndCli(),
					&List,
					bFullPath);
				break;
			case IDM_IMPORTFILE:
				_tcscpy(szTemp, _T(""));
				if (MplayGetaFile(szTemp,FALSE) == MPLAY_ERROR_SUCCESS) {
					DWORD dwInitialListSize = GetListLength(&List);
					MplayInputFile(&List,szTemp);
					MplayRefreshListAppend(GethWndCli(),
						&List,
						bFullPath,
						dwInitialListSize);
				}
				break;
			case IDM_FIND:
				if (MplayFindEntry(&RecentSearches,&dwFindFlags) == MPLAY_ERROR_SUCCESS) {
					LPTSTR szFindText = GetListItem(&RecentSearches, 0);
					int i;
					ASSERT(szFindText != NULL);
					i = (int)SendMessage(GethWndCli(),
						LB_GETCURSEL,
						0,
						0);
					if (i==LB_ERR) {
						i=0;
					}
					i = MplayFindNextText(&List,
						i,
						szFindText,
						dwFindFlags);
					if (i >= 0) {
						SendMessage(GethWndCli(),
							LB_SETCURSEL,
							i,
							0);
					} else {
						MessageBox(hWnd,
							_T("No matches found."),
							_T(MPLAY_NAME),
							MB_ICONINFORMATION);
					}
				}
				break;
			case IDM_FINDNEXT:
				{
					long i = -1;
					LPTSTR szFindText = GetListItem(&RecentSearches, 0);
					if (szFindText != NULL) {
						i=(long)SendMessage(GethWndCli(), LB_GETCURSEL, (WPARAM)0, (LPARAM)0);
						if (i==LB_ERR) {
							i=0;
						} else {
							i++;
						}
						if (dwFindFlags & FIND_STARTTOP) dwFindFlags-=FIND_STARTTOP;
						i = MplayFindNextText(&List,
							i,
							szFindText,
							dwFindFlags);
					}
					if (i>=0) {
						SendMessage(GethWndCli(),
							LB_SETCURSEL,
							(WPARAM)i,
							0);
					} else {
						MessageBox(hWnd,
							_T("No more matches found."),
							_T(MPLAY_NAME),
							MB_ICONINFORMATION);
					}
				}
				break;
			case IDM_FINDPREV:
				{
					long i = -1;
					LPTSTR szFindText = GetListItem(&RecentSearches, 0);
					if (szFindText != NULL) {
						DWORD dwLocalFindFlags;
						i = (long)SendMessage(GethWndCli(),
								LB_GETCURSEL,
								(WPARAM)0,
								(LPARAM)0);
						if (i == LB_ERR) {
							i = GetListLength(&List) - 1;
						} else {
							i--;
						}
						if (dwFindFlags & FIND_STARTTOP) dwFindFlags -= FIND_STARTTOP;
						if (i >= 0) {
							dwLocalFindFlags = dwFindFlags ^ FIND_BACKWARDS;
							i = MplayFindNextText(&List,
								i,
								szFindText,
								dwLocalFindFlags);
						}
					}
					if (i >= 0) {
						SendMessage(GethWndCli(),
							LB_SETCURSEL,
							(WPARAM)i,
							0);
					} else {
						MessageBox(hWnd,
							_T("No more matches found."),
							_T(MPLAY_NAME),
							MB_ICONINFORMATION);
					}
				}
				break;
			case IDM_STRIP:
				if (MplayFindEntry(&RecentSearches,&dwFindFlags) == MPLAY_ERROR_SUCCESS) {
					LPTSTR szFindText = GetListItem(&RecentSearches, 0);
					int i;
					ASSERT(szFindText != NULL);
					i = (int)SendMessage(GethWndCli(),
						LB_GETCURSEL,
						0,
						0);
					if (i==LB_ERR) i=0;
					i = MplayFindNextText(&List,
						i,
						szFindText,
						dwFindFlags);
					while (i>=0) {
						MplayDeleteListItem(&List, i);
						i--;
						if (i<0) i=0;
						i = MplayFindNextText(&List,
							i,
							szFindText,
							dwFindFlags);
					}
					MplayRefreshList(GethWndCli(),
						&List,
						bFullPath);
				}
				break;
#ifdef INTERNAL
			case IDM_PROPERTIES:
				{
					LONG i;
					i = (LONG)SendMessage(GethWndCli(),
						LB_GETCURSEL,
						0,
						0);
					if (i >= 0) {
						LPTSTR szListItem = GetListItem(&List, (DWORD)i);
						DialogBoxParam(GethInst(),
							_T("IDDPROP"),
							GethWndMain(),
							(DLGPROC)MplayPropertiesDlgProc,
							(LPARAM)szListItem);
					} else {
						MessageBox(hWnd,
							_T("Please select the track which you would like to obtain Properties for."),
							_T(MPLAY_NAME),
							MB_ICONINFORMATION);
					}
				}
				break;
#endif
			default:
				return DefWindowProc(hWnd,
					message,
					wParam,
					lParam);
				break;
			}
		break;
	case WM_CREATE:
		{
			bFullPath = FALSE;
			bOnTop = FALSE;
			dwRepeat = REPEAT_NEXTTRACK;
			_tcscpy(szOpenFile, _T(""));
			dwFindFlags = 0;
			MplayResetHandlers();
			MplayAddHandler(_T("mp3"), _T("MPEGVideo"));
			MplayInitList(&List);
			MplayInitList(&RecentSearches);
			DragAcceptFiles(hWnd,TRUE);
			MplayAddTrayIcon(hWnd);
		}

		break;
	case C_WM_REALBEGIN:
#if defined(SCRIPTING)&&(!defined(_CYGWIN))
		{
			_tsearchenv(_T("default.msc"), _T("PATH"), szTemp);
			if (szTemp && _tcslen(szTemp)) {
				MplayPlayScript(&List,
					szTemp);
				MplayRefreshList(GethWndCli(),
					&List,
					bFullPath);
			}
		}
#endif
		break;
#ifdef WM_MOUSEWHEEL
	case WM_MOUSEWHEEL:
		PostMessage(GethWndCli(), WM_MOUSEWHEEL, wParam, lParam);
		break;
#endif
	case WM_DROPFILES:
		{
			DWORD i,nFiles;
			HDROP hDrop;
			HANDLE hFind;
			pi_find fd;
			DWORD InitialListSize;
			hDrop = (HDROP)wParam;
			nFiles = DragQueryFile(hDrop, 0xffffffff, NULL, 0);
			InitialListSize = GetListLength(&List);
			for (i = 0; i < nFiles; i++) {
				DragQueryFile(hDrop,
					(int)i,
					szTemp,
					sizeof(szTemp)/sizeof(TCHAR));
				
				hFind = MplayFindFirst(szTemp, &fd);
				MplayFindClose(hFind);
				if (MplayGetAttr(&fd) & FA_SUBDIR) { 
					MplayGenerateTree(&List, szTemp);
				} else { 
					MplayInputFile(&List, szTemp);
				}
			}
			DragFinish(hDrop);
			MplayRefreshListAppend(GethWndCli(),
				&List,
				bFullPath,
				InitialListSize);

		}
		break;
	case WM_CLOSE:
#if !defined(BROKEN_MINIMIZE)
		MplayDispWindow(hWnd, FALSE);
#else
		MplayCloseStream(hWnd);
		DestroyWindow(hWnd);
		PostQuitMessage(0);
#endif
		break;
	case TRAY_CALLBACK:
		MplayTrayCallback(hWnd, wParam, lParam);
		break;
	case WM_DESTROY:
		MplayFreeList(&List);
		MplayDelTrayIcon(hWnd);
		break;
	case MM_MCINOTIFY:
			if (wParam==MCI_NOTIFY_SUCCESSFUL) {
				switch(dwRepeat) {
				case REPEAT_NEXTTRACK:
					if (MplayPlayMediaFile(hWnd, &List, PLAY_NEXT) != MPLAY_ERROR_SUCCESS) {
						MplayCloseStream(hWnd);
					}
					break;
				case REPEAT_THISTRACK:
					if (MplayPlayMediaFile(hWnd, &List, PLAY_SAME) != MPLAY_ERROR_SUCCESS) {
						MplayCloseStream(hWnd);
					}
					break;
				}
			}

		break;
	case C_WM_CMDLINE:
		{
			LPTSTR a;
			HANDLE hFind;
			pi_find fd;
			a = (LPTSTR)lParam;
			if (a[0] == '"') {
				_tcscpy(szTemp, a + 1);
				a = _tcschr(szTemp, _T('"'));
				if (a) a[0]='\0';
			} else {
				_tcscpy(szTemp,(LPTSTR)lParam);
			}
			if ((hFind = MplayFindFirst(szTemp, &fd)) != 0) {
				if (szTemp && _tcslen(szTemp)) {
					MplayFindClose(hFind);
					if (MplayGetAttr(&fd) & FA_SUBDIR) {
						MplayGenerateTree(&List,
							szTemp);
					} else {
						MplayInputFile(&List,
							szTemp);
					}
					MplayRefreshList(GethWndCli(),
						&List,
						bFullPath);
					MplayPlayMediaFile(hWnd,
						&List,
						0);
				}
			}
		}

		break;

	case WM_SIZE:
		MplayResizeCallback(hWnd, wParam, lParam);
		break;
	case WM_TIMER:
		if (MplayIsTracker())
			MplayUpdatePos();
		break;
	case WM_HSCROLL:
		if (LOWORD(wParam)==TB_THUMBTRACK)
			KillTimer(hWnd,TIMER_ID);
		if (LOWORD(wParam)==SB_ENDSCROLL) {
			MplaySetAudioPos(hWnd);
			if (MplayIsPlaying()&&MplayIsTracker())
				SetTimer(hWnd,TIMER_ID,TIMER_TICK,NULL);
			MplayUpdatePauseStatus(hWnd, FALSE);
		}

		break;
	default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return (0);
}

